use crate::cmd::config::app_config;
use crate::connect_or_retry;
use crate::ems::{Consumer, Producer, Service};
use crate::internal::modbus::code::ModbusCode;
use crate::internal::utils;
use async_trait::async_trait;
use log::info;
use std::net::SocketAddr;
use std::sync::Arc;
use std::time::Duration;
use tokio::sync::Mutex;
use tokio_modbus::client::Reader;

///
/// 高特BMS
/// Modbus-TCP
pub struct GoldBms {
    pub id: String,
    pub producer: Producer,
    pub consumer: Arc<Mutex<Consumer>>,
    pub modbus_code: Arc<Mutex<Vec<ModbusCode>>>,
    pub ctx: Arc<Mutex<tokio_modbus::client::Context>>,
}

impl GoldBms {
    pub async fn new(producer: Producer, consumer: Consumer) -> anyhow::Result<Self> {
        let id = utils::generate_random_str(12);
        utils::log::init_log("inpower_iot_mgc_rs::ems::bms::*", "bms/bms.log").await;
        let context = connect_modbus_tcp().await?;
        info!("BMS[{}]初始化成功!", id);
        Ok(GoldBms {
            id,
            producer,
            consumer: Arc::new(Mutex::new(consumer)),
            modbus_code: Arc::new(Default::default()),
            ctx: Arc::new(Mutex::new(context)),
        })
    }
    async fn read_input_register(&self) -> anyhow::Result<()> {
        self.ctx
            .lock()
            .await
            .read_discrete_inputs(0, 10)
            .await??;
        Ok(())
    }
}

#[async_trait]
impl Service for GoldBms {
    async fn south(&self) {
        loop {
            tokio::time::sleep(Duration::from_millis(app_config().emu.bms.interval)).await;
            match self.read_input_register().await {
                Ok(()) => {
                    println!("读取数据成功!");
                }
                Err(e) => {
                    println!("Modbus 读取失败: {:?}，断开连接", e);
                    let mut x = self.ctx.lock().await;
                    *x = connect_modbus_tcp().await.unwrap();
                }
            }
        }
    }

    async fn north(&self) {
    }
}

async fn connect_modbus_tcp() -> anyhow::Result<tokio_modbus::client::Context> {
    let config = &app_config().emu.bms.gold_bms;
    let addr: SocketAddr = format!("{}:{}", config.host, config.port).parse()?;
    connect_or_retry!(tokio_modbus::client::tcp::connect(addr), "ModbusTCP")
}
